/*
 * Copyright (c) 2005 David HENRY
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */
#include<menuet/os.h>
#include<stdio.h>
#include <kosgl.h> //TinyGL
#include <string.h>
#include "md2.h"

vec3_t anorms_table[162] = {
#include "anorms.h"
};

int ReadMD2Model(const char *filename, md2_model_t *mdl)
{
    FILE *fp;

    fp = fopen(filename,"rb");
    if(!fp) return 0;

    // read header 
  	fread (&mdl->header, 1, sizeof (md2_header_t), fp);


     if ((mdl->header.ident != 844121161) ||
      (mdl->header.version != 8))
     {
      // error! 
      //SysCall(4,(150<<16)+8,0x10ffffff,(int)"error: bad version!",strlen("error: bad version!"));      
	  return 0;
     }   
    
    mdl->frames = new md2_frame_t[mdl->header.num_frames];
        
  // memory allocation 
  mdl->skins = new md2_skin_t[mdl->header.num_skins];
  mdl->texcoords = new md2_texCoord_t[mdl->header.num_st];
  mdl->triangles = new md2_triangle_t[mdl->header.num_tris];
  mdl->frames = new md2_frame_t[mdl->header.num_frames];
  mdl->glcmds = new int[mdl->header.num_glcmds];

  //read model data 
  fseek (fp, mdl->header.offset_skins, SEEK_SET);  
  fread (mdl->skins, sizeof (md2_skin_t), mdl->header.num_skins, fp);

  fseek (fp, mdl->header.offset_st, SEEK_SET);
  fread (mdl->texcoords, sizeof (md2_texCoord_t), mdl->header.num_st, fp);

  fseek (fp, mdl->header.offset_tris, SEEK_SET);
  fread (mdl->triangles, sizeof (md2_triangle_t), mdl->header.num_tris, fp);

  fseek (fp, mdl->header.offset_glcmds, SEEK_SET);
  fread (mdl->glcmds, sizeof (int), mdl->header.num_glcmds, fp);
        
  // read frames 
  fseek (fp, mdl->header.offset_frames, SEEK_SET);
  for (int i = 0; i < mdl->header.num_frames; i++)
    {
      // memory allocation for vertices of this frame 
      mdl->frames[i].verts = new md2_vertex_t[mdl->header.num_vertices];

      // read frame data 
      fread (mdl->frames[i].scale, sizeof (vec3_t), 1, fp);
      fread (mdl->frames[i].translate, sizeof (vec3_t), 1, fp);
      fread (mdl->frames[i].name, sizeof (char), 16, fp);
      fread (mdl->frames[i].verts, sizeof (md2_vertex_t), mdl->header.num_vertices, fp);
    }
    fclose(fp);
return 1;
}


void RenderFrameItpWithGLCmds (int n, float interp, md2_model_t *mdl)
{
  int i, *pglcmds;
  vec3_t v_curr, v_next, v, norm;
  float *n_curr, *n_next;
  md2_frame_t *pframe1, *pframe2;
  md2_vertex_t *pvert1, *pvert2;
  md2_glcmd_t *packet;

  /* check if n is in a valid range */
  if ((n < 0) || (n > (mdl->header.num_frames - 2)))
    return;

  /* enable model's texture */
  glBindTexture (GL_TEXTURE_2D, mdl->tex_id);
  //glEnable(GL_TEXTURE_2D);

  /* pglcmds points at the start of the command list */
  pglcmds = mdl->glcmds;

  /* draw the model */
  while ((i = *(pglcmds++)) != 0)
    {
      if (i < 0)
	{
	  glBegin (GL_TRIANGLE_FAN);
	  i = -i;
	}
      else
	{
	  glBegin (GL_TRIANGLE_STRIP);
	}
      /* draw each vertex of this group */
      for (/* nothing */; i > 0; --i, pglcmds += 3)
	{
	  packet = (md2_glcmd_t *)pglcmds;
	  pframe1 = &mdl->frames[n];
	  pframe2 = &mdl->frames[n + 1];
	  pvert1 = &pframe1->verts[packet->index];
	  pvert2 = &pframe2->verts[packet->index];

	  /* pass texture coordinates to OpenGL */
	  glTexCoord2f (packet->s, packet->t);

	  /* interpolate normals */
	  n_curr = anorms_table[pvert1->normalIndex];
	  n_next = anorms_table[pvert2->normalIndex];

	  norm[0] = n_curr[0] + interp * (n_next[0] - n_curr[0]);
	  norm[1] = n_curr[1] + interp * (n_next[1] - n_curr[1]);
	  norm[2] = n_curr[2] + interp * (n_next[2] - n_curr[2]);

	  glNormal3fv (norm);

	  /* interpolate vertices */
	  v_curr[0] = pframe1->scale[0] * pvert1->v[0] + pframe1->translate[0];
	  v_curr[1] = pframe1->scale[1] * pvert1->v[1] + pframe1->translate[1];
	  v_curr[2] = pframe1->scale[2] * pvert1->v[2] + pframe1->translate[2];

	  v_next[0] = pframe2->scale[0] * pvert2->v[0] + pframe2->translate[0];
	  v_next[1] = pframe2->scale[1] * pvert2->v[1] + pframe2->translate[1];
	  v_next[2] = pframe2->scale[2] * pvert2->v[2] + pframe2->translate[2];

	  v[0] = v_curr[0] + interp * (v_next[0] - v_curr[0]);
	  v[1] = v_curr[1] + interp * (v_next[1] - v_curr[1]);
	  v[2] = v_curr[2] + interp * (v_next[2] - v_curr[2]);

	  glVertex3fv (v);
	}
      glEnd ();      
    }
}
